//Sample using LiquidCrystal library
#include <LiquidCrystal.h>
 
/*******************************************************
 
This program uses the Sainsmart LCD / Keypad shield with 
the Arduino to form a stepper driven dividing head.  The
Sainsmart shield has a 2 row x 16 character LCD for display
and 5 input buttons arranged as left, right, up, down, and
select in addition to a reset button carried up from the
Arduino motherboard.
 
********************************************************/
 
// select the pins used on the LCD panel
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// Arduino pins used for stepper control
#define stepDir 2		//Digital Pin 2 outputs Step Direction
#define stepGo 3		//Digital Pin 3 outputs Step Execution
#define stepDir 12		//Digital Pin 2 outputs Step Direction
#define stepGo 13		//Digital Pin 3 outputs Step Execution 
/************ values used by the panel and buttons*****************/
#define btnRIGHT  0		
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5
#define btnERR    6
int lcd_key  = btnNONE;	//Initialize the key to no button pushed			
int adc_key_in  = 0;			
		
/************* values used by the dividing head *******************/
boolean sel = false;          	// Stores current value of select, toggled true or false
                                        	//  if false, up / down keys increment / decrement delay 			//between steps
                                        	//  if true, up / down keys increment / decrement # of divisions
int del = 5;                            	// default delay value in milliseconds
int divs = 0;                          	// Holds the number of divisions
int pos = 0;                            	// Holds the current dividing head position (0 (zero) based)
int lastpos = 0;		// Last absolute stepper position
int nextstep = 0;                       // Used for calculation of next stepper position
int laststep = 0;		// Holds the last stepper position
int nsteps = 0;                          // Count of steps to advance stepper
int intsteps = 0;
float stepsDiv = 0;		// Steps per Division
float numSteps = 1000.0;          // Holds either Rotary Table or Dividing Head steps per rev
#define stepperDiv 1000.0	 // Stepper Resolution
#define rotaryDiv 14400.0

 
/******************* Move the stepper ****************************/
void step (int dir)
{
  pos = pos + dir;                 	// Increment (or Decrement) position
  if( pos == divs )                     	// If position back at start, set to 0
  {  
    pos = 0;
    lastpos = -1;
  }
  if( pos < 0 )                         	// If position just moved from 0 to -1, set to # of divs - 1
  {
    pos = divs - 1;
    lastpos = divs;
  }    
  laststep = round(stepsDiv * lastpos);
  nextstep = round(stepsDiv * pos);
  nsteps = abs(nextstep - laststep);
  intsteps = nsteps;
  if(dir > 0)                           	// Set the step direction
  {
    digitalWrite(stepDir,LOW);
  }
  else
  {
    digitalWrite(stepDir,HIGH);
  }
  lcd.setCursor(9,0);
  lcd.print(nsteps);
  while(nsteps > 0)                 	// Loop through number of steps
  {
    digitalWrite(stepGo,1);       	// Start Stepper
    delayMicroseconds (200);
    digitalWrite(stepGo,0);       	// Stop Stepper
    delay( del );
    nsteps--;
  }
  lastpos = pos;
}
 
/********************* read the buttons **************************/
int read_LCD_buttons()
{
 adc_key_in = 0;
 do 
 {
   adc_key_in = analogRead(0);      	// read the value from the sensor
 } while( adc_key_in > 1000 );
   lcd.setCursor( 9,0 );
   lcd.print( "       ");
   lcd.setCursor( 9,0 );
   lcd.print( adc_key_in);
 delay(200); 
 // my buttons when read are centered at these valies: 0, 144, 329, 504, 741
 // we add approx 50 to those values and check to see if we are close
 if (adc_key_in < 50)   return btnRIGHT;  
 if (adc_key_in < 195)  return btnUP; 
 if (adc_key_in < 380)  return btnDOWN; 
 if (adc_key_in < 555)  return btnLEFT; 
 if (adc_key_in < 790)  return btnSELECT;   
 return btnERR;  			// when all others fail, return this...
}

/********************* Set Up ************************************/
void setup()
{
 lcd.begin(16, 2);              		// start the library
 do 
 {
   lcd_key = read_LCD_buttons();
   if( lcd_key == btnUP ) numSteps = stepperDiv;
   if( lcd_key == btnDOWN ) numSteps = rotaryDiv;
   lcd.clear();
   lcd.print( numSteps );
 } while( lcd_key != btnSELECT );
 pinMode(stepDir,OUTPUT);
 pinMode(stepGo,OUTPUT);
 lcd.clear();
 lcd.print( "Set Num of Divsions" );
}
  
/********************Main Program Loop ***************************/
void loop()
{
 lcd_key = read_LCD_buttons();  	        // read the buttons
 switch (lcd_key)               	      // depending on which button was pushed, we perform an action
 {
   case btnLEFT:                              // Step dividing head back one interval
     {
       step (-1);
       break;
     }
   case btnRIGHT:                             // Step dividing head forward one interval
     {
     step (1);
     break;
     }
   case btnUP:                                // Increment the number of divisions
     {
     if( sel )
     {  
       del=del+1;
     }
     else
     {     
       divs++;				      
       stepsDiv = numSteps / divs;
     }
     break;
     }
     
   case btnDOWN:                              // Decrement the number of divisions if > 0
   {
     if( sel )
     {
       if( del > 0 )
       {
         del=del-1;
       }
     }
     else
     {
     if(divs > 0) divs --;		      
     stepsDiv = numSteps / divs;
     }
     break;
   }
   case btnSELECT:
     {
     sel = !sel;                              // Toggle Select on or off 
     break;
     }
   case btnERR:
   {
     lcd.clear();
     lcd.print( "Button Error!" );
     return;
   }
 }
 
 /***************** Print all the updated data to the LCD ***************************/
 lcd.clear();
 lcd.print("DIV: ");
 lcd.print(divs);      			        // display the number of divisions
 lcd.setCursor(0,1);
 lcd.print("POS: ");
 lcd.print(pos+1);			        // display the current position
 lcd.setCursor(9,1);
 lcd.print(nextstep);                            // print the stepper position
 lcd.setCursor(14,0);
 if( sel ) 
 {
   lcd.print( "S" );
 }
 else
 {
   lcd.print( " " );
 }
 lcd.setCursor(14,1);
 lcd.print( del );
 }
